@@ -205,9 +205,10 @@ make_stancode <- function(formula, data = NULL, family = gaussian(),
   
   # generate transformed parameters block
   # loop over all observations in transformed parameters if necessary
-  make_loop <- nrow(ee$random) || (Kar || Kma) && !use_cov(autocor) ||  
-               length(intercepts) > 1L || length(ee$mono) ||
-               isTRUE(text_pred$transform) || length(nonlinear)
+  make_loop <- nrow(ee$random) || (Kar || Kma) && !use_cov(autocor) || 
+               is(autocor, "cor_bsts") || length(intercepts) > 1L || 
+               length(ee$mono) || isTRUE(text_pred$transform) || 
+               length(nonlinear)
   if (make_loop && !is_multi) {
     text_loop <- c("  for (n in 1:N) { \n", "  } \n")
   } else if (is_multi) {
@@ -249,7 +250,7 @@ make_stancode <- function(formula, data = NULL, family = gaussian(),
       "  if (!prior_only) { \n  ",
       text_llh, 
       if (needs_lp_pre)
-        "    increment_log_prob(dot_product(weights, lp_pre)); \n",
+        "    target += dot_product(weights, lp_pre); \n",
       "  } \n", 
       text_rngprior$model,
     "} \n")
