@@ -27,7 +27,8 @@
 make_standata <- function(formula, data = NULL, family = "gaussian", 
                           prior = NULL, autocor = NULL, nonlinear = NULL, 
                           partial = NULL, cov_ranef = NULL, 
-                          sample_prior = FALSE, control = NULL, ...) {
+                          sample_prior = FALSE, knots = NULL, 
+                          control = NULL, ...) {
   # internal control arguments:
   #   is_newdata: is make_standata is called with new data?
   #   not4stan: is make_standata called for use in S3 methods?
@@ -35,6 +36,8 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
   #   omit_response: omit checking of the response?
   #   ntrials, ncat, Jm: standata based on the original data
   dots <- list(...)
+  not4stan <- isTRUE(control$not4stan)
+  is_newdata <- isTRUE(control$is_newdata)
   # use deprecated arguments if specified
   cov_ranef <- use_alias(cov_ranef, dots$cov.ranef, warn = FALSE)
   # some input checks 
@@ -54,14 +57,15 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
   ee <- extract_effects(formula, family = family, et$all, 
                         nonlinear = nonlinear)
   prior <- as.prior_frame(prior)
-  check_prior_content(prior, family = family)
-  na_action <- if (isTRUE(control$is_newdata)) na.pass else na.omit
+  check_prior_content(prior, family = family, warn = FALSE)
+  na_action <- if (is_newdata) na.pass else na.omit
   data <- update_data(data, family = family, effects = ee, et$group,
-                      drop.unused.levels = !isTRUE(control$is_newdata),
-                      na.action = na_action, terms_attr = control$terms_attr)
+                      drop.unused.levels = !is_newdata, 
+                      na.action = na_action, knots = knots,
+                      terms_attr = control$terms_attr)
   
   # sort data in case of autocorrelation models
-  if (has_arma(autocor)) {
+  if (has_arma(autocor) || is(autocor, "cor_bsts")) {
     # amend if zero-inflated and hurdle models ever get 
     # autocorrelation structures as they are also using 'trait'
     if (is_forked) {
@@ -133,7 +137,7 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
       if (length(unique(standata$Y)) < 2L) {
         stop("At least two response categories are required.", call. = FALSE)
       }
-    } else if (is.skewed(family)) {
+    } else if (is.skewed(family) || is.lognormal(family)) {
       if (min(standata$Y) <= 0) {
         stop(paste("family", family$family, "requires response variable", 
                    "to be positive"), call. = FALSE)
@@ -157,30 +161,28 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
     }
     standata <- c(standata, list(KC = ncol(C), C = C)) 
     for (i in seq_along(nlpars)) {
-      data_fixef <- data_fixef(ee$nonlinear[[i]], data = data, 
-                               family = family, nlpar = nlpars[i],
-                               not4stan = isTRUE(control$not4stan))
-      data_monef <- data_monef(ee$nonlinear[[i]], data = data, prior = prior, 
+      nle <- ee$nonlinear[[i]]
+      data_fixef <- data_fixef(nle, data = data, family = family, 
+                               nlpar = nlpars[i], knots = knots, 
+                               not4stan = not4stan, G = control$G[[i]])
+      data_monef <- data_monef(nle, data = data, prior = prior, 
                                Jm = control[[paste0("Jm_", nlpars[i])]],
                                nlpar = nlpars[i])
-      data_ranef <- data_ranef(ee$nonlinear[[i]], data = data, 
-                               family = family, cov_ranef = cov_ranef,
-                               is_newdata = isTRUE(control$is_newdata),
-                               not4stan = isTRUE(control$not4stan),
-                               nlpar = nlpars[i])
+      data_ranef <- data_ranef(nle, data = data, family = family, 
+                               cov_ranef = cov_ranef, nlpar = nlpars[i], 
+                               is_newdata = is_newdata, not4stan = not4stan)
       standata <- c(standata, data_fixef, data_monef, data_ranef)
     }
   } else {
     data_fixef <- data_fixef(ee, data = data, family = family, 
-                             not4stan = isTRUE(control$not4stan))
+                             autocor = autocor, knots = knots,
+                             not4stan = not4stan, G = control$G)
     data_monef <- data_monef(ee, data = data, prior = prior, Jm = control$Jm)
     data_csef <- data_csef(ee, data = data)
     data_ranef <- data_ranef(ee, data = data, family = family, 
-                             cov_ranef = cov_ranef,
-                             is_newdata = isTRUE(control$is_newdata),
-                             not4stan = isTRUE(control$not4stan))
+                             cov_ranef = cov_ranef, not4stan = not4stan,
+                             is_newdata = is_newdata)
     standata <- c(standata, data_fixef, data_monef, data_csef, data_ranef)
-    # offsets are not yet implemented for non-linear models
     standata$offset <- model.offset(data)
   }
   # data for specific families
@@ -280,8 +282,9 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
   }
   # autocorrelation variables
   if (has_arma(autocor)) {
-    tgroup <- data[[et$group]]
-    if (is.null(tgroup)) {
+    if (nchar(et$group)) {
+      tgroup <- data[[et$group]]
+    } else {
       tgroup <- rep(1, standata$N) 
     }
     Kar <- get_ar(autocor)
@@ -289,7 +292,7 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
     Karr <- get_arr(autocor)
     if (Kar || Kma) {
       # ARMA effects (of residuals)
-      standata$tg <- as.numeric(as.factor(tgroup))
+      standata$tg <- as.numeric(factor(tgroup))
       standata$Kar <- Kar
       standata$Kma <- Kma
       standata$Karma <- max(Kar, Kma)
@@ -330,6 +333,14 @@ make_standata <- function(formula, data = NULL, family = "gaussian",
     }
     standata$V <- V
   }
+  if (is(autocor, "cor_bsts")) {
+    if (nchar(et$group)) {
+      tgroup <- data[[et$group]]
+    } else {
+      tgroup <- rep(1, standata$N) 
+    }
+    standata$tg <- as.numeric(factor(tgroup))
+  }
   standata$prior_only <- ifelse(identical(sample_prior, "only"), 1L, 0L)
   if (isTRUE(control$save_order)) {
     attr(standata, "old_order") <- attr(data, "old_order")
