@@ -8,17 +8,16 @@ test_that("fitted helper functions run without errors", {
   nsamples <- brms:::Nsamples(fit)
   nobs <- nobs(fit)
   # test preparation of truncated models
-  standata <- standata(fit)
-  standata$lb <- 0
-  standata$ub <- 200
-  mu <- brms:::fitted_response(x = fit, eta = eta, data = standata)
+  draws$data$lb <- 0
+  draws$data$ub <- 200
+  mu <- brms:::fitted_response(draws = draws, mu = eta)
   expect_equal(dim(mu), c(nsamples, nobs))
   # pseudo log-normal model
   fit$family <- gaussian("log")
   expect_equal(dim(fitted(fit, summary = FALSE)), c(nsamples, nobs))
   # pseudo weibull model
   fit$family <- weibull()
-  expect_equal(dim(fitted(fit, summary = FALSE)), c(nsamples, nobs))
+  expect_equal(dim(SW(fitted(fit, summary = FALSE))), c(nsamples, nobs))
   # pseudo binomial model
   fit$autocor <- cor_arma()
   fit$family <- binomial()
@@ -26,6 +25,7 @@ test_that("fitted helper functions run without errors", {
   # pseudo hurdle poisson model
   names(fit$data)[1] <- "response"
   fit$family <- hurdle_poisson()
+  fit$formula <- count ~ Trt*Age + mono(Exp) + offset(Age) + (1+Trt|visit)
   expect_equal(dim(fitted(fit, summary = FALSE)), c(nsamples, nobs(fit)))
   # pseudo zero-inflated poisson model
   fit$family <- zero_inflated_poisson()
@@ -35,30 +35,31 @@ test_that("fitted helper functions run without errors", {
                           max_obs = 4, family = cumulative())
   expect_equal(dim(mu), c(nsamples, nobs, 4))
   # truncated continous models
-  data <- list()
-  mu <- fitted_trunc_gaussian(eta, lb = 0, ub = 10, x = fit, data = data)
+  draws$nu <- c(posterior_samples(fit, pars = "^nu$", as.matrix = TRUE))
+  draws$shape <- c(posterior_samples(fit, pars = "^shape$", as.matrix = TRUE))
+  mu <- fitted_trunc_gaussian(eta, lb = 0, ub = 10, draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
-  mu <- fitted_trunc_student(eta, lb = -Inf, ub = 15, x = fit, data = data)
+  mu <- fitted_trunc_student(eta, lb = -Inf, ub = 15, draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
-  mu <- fitted_trunc_lognormal(eta, lb = 2, ub = 15, x = fit, data = data)
+  mu <- fitted_trunc_lognormal(eta, lb = 2, ub = 15, draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
   exp_eta <- exp(eta)
-  mu <- fitted_trunc_gamma(exp_eta, lb = 1, ub = 7, x = fit)
+  mu <- fitted_trunc_gamma(exp_eta, lb = 1, ub = 7, draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
-  mu <- fitted_trunc_exponential(exp_eta, lb = 0, ub = Inf, x = fit)
+  mu <- fitted_trunc_exponential(exp_eta, lb = 0, ub = Inf, draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
-  mu <- fitted_trunc_weibull(exp_eta, lb = -Inf, ub = Inf, x = fit)
+  mu <- SW(fitted_trunc_weibull(exp_eta, lb = -Inf, ub = Inf, draws = draws))
   expect_equal(dim(mu), c(nsamples, nobs))
   # truncated discrete models
   data <- list(Y = sample(100, 10), trials = 1:10, N = 10)
-  mu <- fitted_trunc_poisson(exp_eta, lb = 0, ub = 100, data = data)
+  mu <- fitted_trunc_poisson(exp_eta, lb = 0, ub = 100, draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
-  mu <- fitted_trunc_negbinomial(exp_eta, lb = 0, ub = 100, x = fit, 
-                                 data = data)
+  mu <- fitted_trunc_negbinomial(exp_eta, lb = 0, ub = 100, draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
-  mu <- fitted_trunc_geometric(exp_eta, lb = 0, ub = 100, data = data)
+  mu <- fitted_trunc_geometric(exp_eta, lb = 0, ub = 100, draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
+  draws$data$trials <- 120
   mu <- fitted_trunc_binomial(ilink(eta, "logit"), lb = -Inf, ub = 100, 
-                              data = data)
+                              draws = draws)
   expect_equal(dim(mu), c(nsamples, nobs))
 })
\ No newline at end of file
