@@ -95,4 +95,11 @@ test_that("use_alias works correctly", {
 test_that("rhs keeps attributes", {
   form <- structure(y~x, test = TRUE)
   expect_equal(attributes(form), attributes(rhs(form)))
+})
+
+test_that("lsp works correctly", {
+  expect_equal(lsp("base", pattern = "^log"),
+               c("log", "log10", "log1p", "log2", "logb", "logical"))
+  expect_equal(lsp("brms", pattern = "^log_"),
+               c("log_diff_exp", "log_inv_logit", "log_sum_exp"))
 })
\ No newline at end of file
